@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -290,10 +290,7 @@ def _create(path):
         0700. If the destination directory exists already the permissions
         should not be changed.
     """
-    try:
-        os.makedirs(path, 0o700)
-    except FileExistsError:
-        pass
+    os.makedirs(path, 0o700, exist_ok=True)
 
 
 def _init_dirs(args=None):
